"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const nodes_1 = require("./nodes");
function parse(source) {
    let sourceFile = ts.createSourceFile("code.ts", source, ts.ScriptTarget.Latest, true);
    let tree = createSourceTree(sourceFile, source)[0];
    return findTopTree(tree);
}
exports.parse = parse;
function findTopTree(tree) {
    let childrenWithoutSpaces = tree.children.filter(child => !(child instanceof nodes_1.Space));
    if (childrenWithoutSpaces.length !== 1) {
        return tree;
    }
    let onlyChild = childrenWithoutSpaces[0];
    if (onlyChild instanceof nodes_1.Tree) {
        return findTopTree(onlyChild);
    }
    else {
        return tree;
    }
}
function createSourceTree(tsNode, fullCode, codePosition = {
        charIndex: 0
    }) {
    if (!ts.isToken(tsNode)) {
        let tree = new nodes_1.Tree(tsNode.kind);
        for (let child of tsNode.getChildren()) {
            let childNodes = createSourceTree(child, fullCode, codePosition);
            let firstChildNode = childNodes[0];
            if (firstChildNode instanceof nodes_1.Tree) {
                while (firstChildNode.children[0] instanceof nodes_1.Space) {
                    tree.children.push(firstChildNode.children[0]);
                    firstChildNode.children = firstChildNode.children.slice(1);
                }
            }
            tree.children.push(...childNodes);
        }
        return [tree];
    }
    else {
        let nodes = [];
        let fullText = tsNode.getFullText();
        let tokenText = tsNode.getText();
        if (fullText.trim().length === 0) {
            nodes.push(new nodes_1.Space(fullText));
            codePosition.charIndex += fullText.length;
        }
        else {
            let tokenPositionInFullText = fullText.indexOf(tokenText);
            if (tokenPositionInFullText > 0) {
                nodes.push(new nodes_1.Space(fullText.substr(0, tokenPositionInFullText)));
                codePosition.charIndex += tokenPositionInFullText;
            }
            nodes.push(new nodes_1.Token(tsNode.kind, tokenText));
            codePosition.charIndex += tokenText.length;
            if (tokenPositionInFullText + tokenText.length !== fullText.length) {
                let remainingText = fullText.substr(tokenPositionInFullText + tokenText.length);
                nodes.push(new nodes_1.Space(remainingText));
                codePosition.charIndex += remainingText.length;
            }
        }
        return nodes;
    }
}
